@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *    Copyright (c) 2004-2020 by the GMT Team (https://www.generic-mapping-tools.org/team.html) and Michael Chandler
+ *    Copyright (c) 2004-2021 by the GMT Team (https://www.generic-mapping-tools.org/team.html) and Michael Chandler
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /*
@@ -73,23 +73,28 @@ static int usage (struct GMTAPI_CTRL *API, int level) {
 
 	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: %s <cruise(s)>  [-H<headinfo>] [-Mf[<item>]|r|e|h] [%s] [%s]\n\n", name, GMT_V_OPT, GMT_PAR_OPT);
+	GMT_Usage (API, 0, "usage: %s <cruise(s)>  [-H<headertable>] [-Mf[<item>]|r|t] [%s] [%s]\n", name, GMT_V_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
 	MGD77_Init (API->GMT, &M);		/* Initialize MGD77 Machinery */
+	GMT_Message (API, GMT_TIME_NONE, "  REQUIRED ARGUMENTS:\n");
 	MGD77_Cruise_Explain (API->GMT);
-	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-H Read and assign header values from a file. Each input file row gives an exact\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   header_field_name, space or tab, and header value. Values are read according to\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   NGDC's MGD77 header format specification.\n\t\te.g.,\n\t\tSource_Institution Univ. of Hawaii\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t\tPort_of_Arrival Honolulu, HAWAII\n\t\t...\n	   See mgd77info -Mf output for recognized header field names.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-M Print header items.  Append type of presentation:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     f: Print header items individually, one per line.  Append name of a particular\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t        item (e.g., Port_of_Departure), all [Default], or - to see a list of items.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t        You can also use the number of the item.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     r: Display raw original MGD77 header records [Default].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     t: Display raw original M77T header records.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\n  OPTIONAL ARGUMENTS:\n");
+	GMT_Usage (API, 1, "\n-H<headertable>");
+	GMT_Usage (API, -2, "Read and assign header values from the given file. Each input file row gives an exact "
+		"header_field_name, space or tab, and header value. Values are read according to "
+		"NGDC's MGD77 header format specification. Two examples are given:");
+	GMT_Usage (API, 3, "Source_Institution Univ. of Hawaii.");
+	GMT_Usage (API, 3, "Port_of_Arrival Honolulu, HAWAII");
+	GMT_Usage (API, -2, "Note: See mgd77info -Mf output for recognized header field names.");
+	GMT_Usage (API, 1, "\n-Mf[<item>]|r|t");
+	GMT_Usage (API, -2, "Print header items.  Append type of presentation:");
+	GMT_Usage (API, 3, "f: Print header items individually, one per line.  Append name of a particular "
+		"item (e.g., Port_of_Departure), all [Default], or - to see a list of items."
+		"You can also use the number of the item.");
+	GMT_Usage (API, 3, "r: Display raw original MGD77 header records [Default].");
+	GMT_Usage (API, 3, "t: Display raw original M77T header records.");
 	GMT_Option (API, "V,.");
 
 	MGD77_end (API->GMT, &M);	/* Close machinery */
@@ -119,11 +124,13 @@ static int parse (struct GMT_CTRL *GMT, struct MGD77HEADER_CTRL *Ctrl, struct GM
 			/* Processes program-specific parameters */
 
 			case 'H':
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->H.active);
 				Ctrl->H.active = true;
 				Ctrl->H.file = strdup (opt->arg);
 				break;
 
 			case 'M':
+				n_errors += gmt_M_repeated_module_option (API, Ctrl->M.active);
 				Ctrl->M.active = true;
 				if (opt->arg[0] == 'f') {
 					Ctrl->M.mode = FORMATTED_HEADER;
@@ -225,12 +232,13 @@ EXTERN_MSC int GMT_mgd77header (void *V_API, int mode, void *args) {
 	if (n_paths <= 0) {
 		GMT_Report (API, GMT_MSG_ERROR, "No cruises given\n");
 		if (fp) gmt_fclose (GMT, fp);
+		MGD77_Path_Free (GMT, (uint64_t)n_paths, list);
 		Return (GMT_NO_INPUT);
 	}
 
 	saved_range = GMT->current.io.geo.range;	/* We may have to reset thisso keep a copy */
 	gmt_set_geographic (GMT, GMT_OUT);	/* Output lon/lat */
-	gmt_set_column (GMT, GMT_OUT, GMT_Z, M.time_format);
+	gmt_set_column_type (GMT, GMT_OUT, GMT_Z, M.time_format);
 
 	use = (M.original || M.format != MGD77_FORMAT_CDF) ? MGD77_ORIG : MGD77_REVISED;
 
@@ -244,6 +252,8 @@ EXTERN_MSC int GMT_mgd77header (void *V_API, int mode, void *args) {
 
 		if (MGD77_Read_File_nohdr (GMT, list[argno], &M, D)) {
 			GMT_Report (API, GMT_MSG_ERROR, "Failure while reading data for cruise %s\n", list[argno]);
+			MGD77_Free_Dataset (GMT, &D);	/* Free memory allocated by MGD77_Read_File */
+			MGD77_Path_Free (GMT, (uint64_t)n_paths, list);
 			Return (GMT_DATA_READ_ERROR);
 		}
 
@@ -377,6 +387,8 @@ EXTERN_MSC int GMT_mgd77header (void *V_API, int mode, void *args) {
 
 		if (gmt_M_is_dnan(tmin) || gmt_M_is_dnan(tmax)) {
 			GMT_Report (API, GMT_MSG_ERROR, "Abort: cruise %s no time records\n", M.NGDC_id);
+			MGD77_Free_Dataset (GMT, &D);
+			MGD77_Close_File (GMT, &M);
 			Return (GMT_DATA_READ_ERROR);
 		}
 		MGD77_Close_File (GMT, &M);
